﻿/*global $, picolour*/
$(function () {
	'use strict';
	var numChannels,
		$colourPicker = $('#colour_picker'),
		$tabs = $('#tabs'),
		setTabColour = function (colour) {
			$colourPicker.hsvpicker('setColour', colour);
		},
		landscapeQuery = window.matchMedia('(orientation: landscape)'),
		setOrientation = function (mql) {
			var orientation = mql.matches ? 'landscape' : 'portrait';

			$colourPicker.hsvpicker('setOrientation', orientation);
			$tabs.scalingTabs('setOrientation', orientation);
		},
		toggleTimeInputs = function($select) {
			var selectedIndex = $select.find('option:selected').index();

			$select.closest('div').find('input').parent().hide().eq(selectedIndex).show();
		},
		toggleLoading = function (loading) {
			$('#loading').toggle(loading);
			$('#container').toggle(!loading);
		},
		refreshConfigTab = function () {
			picolour.webIoPi.get('override', function (data) {
				$('input[name=enable][value='+data+']').prop('checked',true);
			});
			
			picolour.webIoPi.get('fadeDuration', function (data) {
				$('input[name=fade_duration]').val(data);
			});

			picolour.webIoPi.get('location', function (data) {
				$('input[name=latitude]').val(data.latitude);
				$('input[name=longitude]').val(data.longitude);
			});

			picolour.webIoPi.get('onTime', function (data) {
				$('select[name=on_time]').val(data.type);
				$('input[name=on_time_offset]').val(data.offset);
				$('input[name=on_time_time]').val(data.time);
				toggleTimeInputs($('select[name=on_time]'));
			});

			picolour.webIoPi.get('offTime', function (data) {
				$('select[name=off_time]').val(data.type);
				$('input[name=off_time_offset]').val(data.offset);
				$('input[name=off_time_time]').val(data.time);
				toggleTimeInputs($('select[name=off_time]'));
			});
		};

	toggleLoading(true);

	$colourPicker.hsvpicker({
		changePeriodMin: 100,
		change: function (newColour) {
			var channel = $tabs.scalingTabs('getSelectedIndex');

			picolour.webIoPi.set('colour', newColour, channel);
		}
	});

	$tabs.scalingTabs({
		changeTab: function (index) {
			if (index < numChannels) {
				//TODO: Also get colour on a timer, in case someone else is fiddling at the same time.
				picolour.webIoPi.get('colour', setTabColour, index);
			} else {
				refreshConfigTab();
			}
		}
	});

	//TODO: If loading fails, offer "retry" button.
	picolour.webIoPi.get('channels', function (data) {
		numChannels = data.length;

		$.each(data, function (index, element) {
			element.href = '#colour_picker';
			$tabs.scalingTabs('addTab', element);
		});
		//TODO: Maybe allow tabs to be inserted at specific indexes, so the config tab can be inserted earlier?
		$tabs.scalingTabs('addTab', {
			name: '★',
			title: 'Configuration',
			href: '#config'
		});

		toggleLoading(false);
	}, 'json');

	setOrientation(landscapeQuery);
	landscapeQuery.addListener(setOrientation);

	//Setup config tab.
	$('input[name=enable]').on('change', function () {
		picolour.webIoPi.set('override', $(this).val());
	});

	$('input[name=fade_duration]').on('change', function () {
		picolour.webIoPi.set('fadeDuration', parseFloat($(this).val()));
	});

	$('input[name=latitude], input[name=longitude]').on('change', function () {
		if (this.checkValidity()) {
			picolour.webIoPi.set('location', {
				latitude: parseFloat($('input[name=latitude]').val()),
				longitude: parseFloat($('input[name=longitude]').val())
			});
		}
	});

	$('select[name=on_time], input[name=on_time_offset], input[name=on_time_time]').on('change', function() {
		var $this = $(this);

		if ($this.is('select')) {
			//TODO: What if an input contains an invalid value when it is hidden?
			toggleTimeInputs($this);
		}

		if (this.checkValidity()) {
			//TODO: Should check all the inputs for validity, not just this.
			picolour.webIoPi.set('onTime', {
				type: $('select[name=on_time]').val(),
				offset: parseInt($('input[name=on_time_offset]').val()),
				time: $('input[name=on_time_time]').val()
			});
		}

	}).closest('div').find('input').parent().hide();

	$('select[name=off_time], input[name=off_time_offset], input[name=off_time_time]').on('change', function() {
		var $this = $(this);

		if ($this.is('select')) {
			toggleTimeInputs($this);
		}

		if (this.checkValidity()) {
			picolour.webIoPi.set('offTime', {
				type: $('select[name=off_time]').val(),
				offset: parseInt($('input[name=off_time_offset]').val()),
				time: $('input[name=off_time_time]').val()
			});
		}

	}).closest('div').find('input').parent().hide();

	refreshConfigTab();
});