/*global $, picolour*/
$(function() {
	'use strict';
	var hsvEqual = function (hsv1, hsv2) {
			return hsv1.h === hsv2.h && hsv1.s === hsv2.s && hsv1.v === hsv2.v;
		},
		hsvpicker = function ($container, options) {
			var $handleTemplate = $('<div>').addClass('handle')
					.append($('<div>').addClass('marker')),
				$sliderTemplate = $('<div>').addClass('slider')
					.append($('<div>').addClass('gradient')
						.append($handleTemplate.clone())),
				$rotatorTemplate = $('<div>').addClass('rotator')
					.append($('<div>').addClass('wheel')
						.append($handleTemplate.clone())),
				$hPicker = $rotatorTemplate.clone().addClass('h_picker'),
				$hHandle = $hPicker.find('.handle'),
				$sPicker = $sliderTemplate.clone().addClass('s_picker'),
				$sGradient = $sPicker.find('.gradient'),
				$sHandle = $sPicker.find('.handle'),
				$vPicker = $sliderTemplate.clone().addClass('v_picker'),
				$vHandle = $vPicker.find('.handle'),
				moving,
				isTouch,
				currentHsv = { h: 0.0, s: 1.0, v: 0.5 },
				lastHsv = { h: 0.0, s: 1.0, v: 0.5 },
				changeId,
				lastChangeTime,
				landscape = (options.orientation === 'landscape'),
				//Gets the coordinates relative to the target element's centre, in pixels.
				getCoordsFromEvent = function (event) {
					var eventX,
						eventY,
						x,
						y;

					if (event.originalEvent.changedTouches) {
						eventX = event.originalEvent.changedTouches[0].pageX;
						eventY = event.originalEvent.changedTouches[0].pageY;
					} else {
						eventX = event.pageX;
						eventY = event.pageY;
					}

					x = eventX - (event.currentTarget.offsetLeft + (event.currentTarget.offsetWidth / 2));
					y = (event.currentTarget.offsetTop + (event.currentTarget.offsetHeight / 2)) - eventY;

					return { x: x, y: y };
				},
				//Angle relative to target element's centre, in radians.
				getAngleFromEvent = function (event) {
					var coords = getCoordsFromEvent(event),
						theta = Math.atan2(coords.x, coords.y);

					return theta >= 0.0 ? theta : theta + (2.0 * Math.PI);
				},
				//Gets the position relative to the target element's bottom left corner, as a proportion of its size.
				getPositionFromEvent = function (event) {
					var coords = getCoordsFromEvent(event),
						width = event.currentTarget.offsetWidth,
						height = event.currentTarget.offsetHeight,
						xFromOrigin = coords.x + (width / 2.0),
						yFromOrigin = coords.y + (height / 2.0),
						x = xFromOrigin < 0 ? 0 : (xFromOrigin > width ? width : xFromOrigin),
						y = yFromOrigin < 0 ? 0 : (yFromOrigin > height ? height : yFromOrigin);

					return { x: x / width, y: y / height };
				},
				setSlider = function ($slider, value) {
					var percentTranslation = (value * 100.0),
						css;

					if (landscape) {
						css = { 'top': (100.0 - percentTranslation) + '%', 'left': 0 };
					} else {
						css = { 'top': 0, 'left': percentTranslation + '%' };
					}

					$slider.css(css);
				},
				setH = function (h) {
					var transformValue = 'rotate(' + h + 'rad)',
						//Set colour in HSL because it's very easy to calculate with S and L being fixed.
						colourString = 'hsl(' + (h / (2.0 * Math.PI)) * 360.0 + ', 100%, 50%)';

					$hHandle.css({ '-webkit-transform': transformValue, 'transform': transformValue });
					$sGradient.css('background-image', '-webkit-linear-gradient(' + (landscape ? 'bottom' : 'left') + ', #FFFFFF, ' + colourString + ')')
						.css('background-image', 'linear-gradient(' + (landscape ? '0deg' : '90deg') + ', #FFFFFF, ' + colourString + ')');
					currentHsv.h = h;
				},
				setS = function (s) {
					setSlider($sHandle, s);
					currentHsv.s = s;
				},
				setV = function (v) {
					setSlider($vHandle, v);
					currentHsv.v = v;
				},
				setHsv = function (hsv) {
					setH(hsv.h);
					setS(hsv.s);
					setV(hsv.v);
				},
				doChange = function () {
					if (hsvEqual(lastHsv, currentHsv)) {
						return;
					}

					var nextChangeDelay;

					lastHsv = $.extend(lastHsv, currentHsv);

					if (options.change) {
						if(!lastChangeTime || (Date.now() - lastChangeTime > options.changePeriodMin)) {
							nextChangeDelay = 0;
						} else {
							nextChangeDelay = (lastChangeTime + options.changePeriodMin) - Date.now();
						}

						if(changeId) {
							clearTimeout(changeId);
						}

						changeId = setTimeout(function () {
							options.change($.extend({}, currentHsv));
							lastChangeTime = Date.now();
							changeId = undefined;
						}, nextChangeDelay);
					}
				},
				setOrientation = function (orientation) {
					landscape = (orientation === 'landscape');
					$container.toggleClass('portrait', !landscape);
					$container.toggleClass('landscape', landscape);
					setHsv(currentHsv);
				},
				handleRotatorInput = function (event, $rotator, setter) {
					var angle;

					event.currentTarget = $rotator.get(0);
					angle = getAngleFromEvent(event);
					moving = $rotator;
					setter(angle);
				},
				handleSliderInput = function (event, $slider, setter) {
					var position;

					event.currentTarget = $slider.get(0);
					position = getPositionFromEvent(event);
					moving = $slider;
					setter(landscape ? position.y : position.x);
				};

			$container.append($hPicker);
			$container.append($sPicker);
			$container.append($vPicker);

			//Mouse and touch work slightly differently: mouse input is ended when the button is released or the mouse leaves the control.
			//Touch input continues until released, even if dragged outside the control.
			//TODO: Should they be more consistent?
			$container.on('mousedown touchstart', '.h_picker, .s_picker, .v_picker', function (event) {
				if ((event.which === 1 || event.type !== 'mousedown') && !moving) {
					event.preventDefault();
					//Record if this is a touch event, to catch unexpected mousemove event that happens on Android when touching and holding.
					isTouch = (event.type === 'touchstart');
					if (event.currentTarget === $hPicker.get(0)) {
						handleRotatorInput(event, $hPicker, setH);
					} else if (event.currentTarget === $sPicker.get(0)) {
						handleSliderInput(event, $sPicker, setS);
					} else if (event.currentTarget === $vPicker.get(0)) {
						handleSliderInput(event, $vPicker, setV);
					}

					doChange();
				}
			}).on('mousemove touchmove', function (event) {
				if (moving && (!isTouch || event.type === 'touchmove')) {
					event.preventDefault();
					if (moving === $hPicker) {
						handleRotatorInput(event, $hPicker, setH);
					} else if (moving === $sPicker) {
						handleSliderInput(event, $sPicker, setS);
					} else if (moving === $vPicker) {
						handleSliderInput(event, $vPicker, setV);
					}

					doChange();
				}
			}).on('mouseup touchend mouseleave', function(event) {
				if (moving) {
					event.preventDefault();
					moving = undefined;
				}
			});

			setOrientation(options.orientation);

			return {
				setHsv: setHsv,
				setOrientation: setOrientation
			};
		},
		//Very simple tabs. Note that the CSS is in % wherever possible, so it all scales perfectly to fit different phone screen sizes.
		scalingTabs = function ($container, options) {
			var landscape = (options.orientation === 'landscape'),
				$tabList = $('<ul>'),
				getSelectedTab = function () {
					return $tabList.find('.selected');
				},
				getSelectedIndex = function () {
					return getSelectedTab().index();
				},
				setOrientation = function (orientation) {
					landscape = (orientation === 'landscape');
					$container.toggleClass('portrait', !landscape);
					$container.toggleClass('landscape', landscape);
				},
				addTab = function (tabOptions) {
					var $newTab = $('<li>')
						.append($('<a>')
							.attr('href', tabOptions.href)
							.attr('title', tabOptions.description)
							.text(tabOptions.name));

					$tabList.append($newTab);

					//Start with first tab selected.
					if ($newTab.index() === 0) {
						$newTab.children().click();
					} else if ($tabList.children('.selected').children('a').attr('href') !== tabOptions.href) {
						$(tabOptions.href).hide();
					}
				};

			$container.addClass('scaling-tabs')
				.append($tabList)
				.on('click', 'a', function (event) {
					var $this = $(this),
						$selectedTab = $this.parent(),
						$previousTab = getSelectedTab(),
						tabId = $this.attr('href');

					event.preventDefault();
					$tabList.children().removeClass('selected');
					$selectedTab.addClass('selected');

					$tabList.children().each(function (index, li) {
						var otherId = $(li).children('a').attr('href');

						if (otherId !== tabId) {
							$(otherId).hide();
						}
					});

					$(tabId).show();

					if (options.changeTab) {
						options.changeTab($selectedTab.index(), $previousTab.index());
					}
				});

			setOrientation(options.orientation);

			return {
				getSelectedIndex: getSelectedIndex,
				setOrientation: setOrientation,
				addTab: addTab
			};
		};

	window.picolour = $.extend(window.picolour, {
		hsvEqual: hsvEqual
	});

	$.fn.hsvpicker = function (method, data) {
		switch (method) {
			case 'setColour':
				this.each(function() {
					$(this).data('hsvpicker').setHsv(data);
				});
				break;
			case 'setOrientation':
				this.each(function() {
					$(this).data('hsvpicker').setOrientation(data);
				});
				break;
			default:
				this.each(function() {
					var picker = hsvpicker($(this), method);
					$(this).data('hsvpicker', picker);
				});
				break;
		}

		return this;
	};

	$.fn.scalingTabs = function (method, data) {
		switch (method) {
			case 'getSelectedIndex':
				return $(this).data('scalingTabs').getSelectedIndex();
			case 'setOrientation':
				this.each(function() {
					$(this).data('scalingTabs').setOrientation(data);
				});
				break;
			case 'addTab':
				this.each(function() {
					$(this).data('scalingTabs').addTab(data);
				});
				break;
			default:
				this.each(function() {
					var tabs = scalingTabs($(this), method);
					$(this).data('scalingTabs', tabs);
				});
				break;
		}

		return this;
	};
});