/*global $, picolour*/
$(function() {
	'use strict';
	var makeUrl = function (path) {
			return '/' + path.join('/');
		},
		currentColour = [],
		isSettingColour = [],
		queuedColour = [],
		queueDelay = 100,
		get = function (url, callback) {
			return $.get(url, function (data) {
				callback(data);
			}, 'json');
		},
		set = function (url, data) {
			return $.ajax({
				url: url,
				type: 'PUT',
				contentType: 'application/json; charset=UTF-8',
				data: JSON.stringify(data)
			});
		},
		//get/setColour are handled specially, to prevent redundant requests when the colour is changed quickly.
		//Note that callback is called twice: immediately with the last known value, and then again when a value is received from the server.
		getColour = function (callback, channel) {
			//TODO: Don't really need to call this twice.
			callback($.extend({}, currentColour));

			return get(makeUrl(['colour', channel]), function (data) {
				currentColour[channel] = data;
				callback($.extend({}, currentColour[channel]));
			});
		},
		setColour = function (newColour, channel) {
			if (!currentColour[channel] || picolour.hsvEqual(currentColour[channel], newColour)) {
				return;
			}

			if (isSettingColour[channel]) {
				if (queuedColour[channel]) {
					queuedColour[channel] = newColour;
				} else {
					queuedColour[channel] = newColour;

					setTimeout(function () {
						var temp = queuedColour[channel];
						queuedColour[channel] = undefined;
						setColour(temp);
					}, queueDelay);
				}

				return;
			}

			currentColour[channel] = newColour;
			isSettingColour[channel] = true;

			return set(makeUrl(['colour', channel]), newColour).always(function () {
				isSettingColour[channel] = false;
			});
		};

	window.picolour = $.extend(window.picolour, {
		webIoPi: {
				get: function (name, callback, channel) {
					return name === 'colour'
						? getColour(callback, channel)
						: get(makeUrl([name]), callback);
				},
				set: function (name, value, channel) {
					return name === 'colour'
						? setColour(value, channel)
						: set(makeUrl([name]), value);
				}
			}
	});
});